/*
  Module Hardware
  part of Arduino Mega Server project
*/

// Memory

// Flash
#define MAX_FLASH_MEMORY 131072


#define USED_FLASH_MEMORY 87592 // add by hands after sketch compilation
#define FREE_FLASH_MEMORY MAX_FLASH_MEMORY - USED_FLASH_MEMORY
#define FREE_FLASH_MEMORY_PERC (FREE_FLASH_MEMORY) / (MAX_FLASH_MEMORY / 100)
// SRAM
#define MAX_MEMORY 29915 // 32768
#define ONE_PERC (MAX_MEMORY / 100)

// hardware structure
typedef struct Hardware {
  byte platform  = 0;
  byte ethernet  = 0;
  byte sdType    = 0;
  byte sdFs      = 0;
  int  sdVolume  = 0;
  byte nrf24L01  = 0;
  byte nooMt1132 = 0;
  byte nooMr1132 = 0;
  byte rtc       = 0;
} hardware;

hardware hard;


void hardwareInit() {
  initStart("Hardware");
  Serial.print(F(" Network adapter:   ")); Serial.print(F("Ethernet Shield ")); Serial.println(ETHERNET_SHIELD);
  Serial.print(F(" Free FLASH memory: ")); Serial.print(FREE_FLASH_MEMORY); Serial.print(" ("); Serial.print(FREE_FLASH_MEMORY_PERC); Serial.println("%)");
  Serial.print(F(" Free SRAM memory:  ")); Serial.print(freeMem());         Serial.print(" ("); Serial.print(freeMemP());             Serial.println("%)");
  initDone();
}

// Memory

extern "C" char* sbrk(int incr);
void printFreeMem() {
  Serial.print(freeMem());
  Serial.print(F(" (")); Serial.print(freeMemP()); Serial.print(F("%)"));
}

int freeMem() {
  char top;
  return &top - reinterpret_cast<char*>(sbrk(0));
}

int freeMemP() {
  return freeMem() / ONE_PERC;
}

// CPU load / Cyclos in second

int summCyclosInSec = 0;
int cyclosInSec = 0;

void cyclosInSecCalc() {
  cyclosInSec = (100 - summCyclosInSec - 5) * 1.1;
  if (cyclosInSec < 0) {cyclosInSec = 0;}
  if (cyclosInSec > 100) {cyclosInSec = 100;}
  summCyclosInSec = 0;
}

void cyclosInSecWork() {
  summCyclosInSec++;
  if (cycle1s) {cyclosInSecCalc();}
}

// CPU load / Cyclos delay

unsigned long pointCpu = 0;
unsigned long oldPointCpu = 0;
unsigned long deltaCpu = 0;
byte const MAX_CPU_BUFFER = 10;
unsigned long bufferCpuLoad[MAX_CPU_BUFFER];
int cyclosDelay = 0;

void shiftCpuLoad(unsigned long tick) {
  for (byte i = MAX_CPU_BUFFER - 1; i > 0; i--) {
    bufferCpuLoad[i] = bufferCpuLoad[i - 1];
  }
  bufferCpuLoad[0] = tick;
}

void averageCpuLoad() {
  unsigned long summ = 0;
  
  for (byte i = 0; i < MAX_CPU_BUFFER; i++) {
    summ += bufferCpuLoad[i];
  }
  cyclosDelay = summ / MAX_CPU_BUFFER;
}

void cyclosDelayWork() {
  pointCpu = millis();
  deltaCpu = pointCpu - oldPointCpu;
  oldPointCpu = pointCpu;
  shiftCpuLoad(deltaCpu);
  averageCpuLoad();
  cyclosDelay -= 8;
  if (cyclosDelay < 0) {cyclosDelay = 0;}
  if (cyclosDelay > 100) {cyclosDelay = 100;}
}

// Works

void hardWorks() {

}
